#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-06-10 19:49:35
@File: swanboard/run/utils.py
@IDE: vscode
@Description:
    启动服务相关的工具函数
"""
from typing import MutableMapping, Optional
import os
from swanboard.utils import FONT
import psutil
import socket
import re

# ---------------------------------- 格式检查 ----------------------------------


def is_ipv4(string: str) -> bool:
    """判断字符串是否是一个ipv4地址

    Parameters
    ----------
    string : str
        待检查的字符串

    Returns
    -------
    bool
        如果是ipv4地址，返回True，否则返回False
    """
    pattern = re.compile(r"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$")
    return isinstance(string, str) and pattern.match(string)


def is_port(string: str) -> bool:
    """判断字符串是否是一个端口号

    Parameters
    ----------
    string : str
        待检查的字符串

    Returns
    -------
    bool
        如果是端口号，返回True，否则返回False
    """
    if not is_int(string):
        return False
    port = int(string)
    return 0 <= port <= 65535


def is_int(string: str) -> bool:
    """判断字符串是否可以转换为整数

    Parameters
    ----------
    string : str
        待检查的字符串

    Returns
    -------
    bool
        如果可以转换为整数，返回True，否则返回False
    """
    try:
        int(string)
        return True
    except ValueError:
        return False


# ---------------------------------- 工具类 ----------------------------------


class URL(object):
    # 生成链接提示,先生成各个组件
    _arrow = "\t\t\t➜"
    arrow = FONT.bold(FONT.green(_arrow))
    local = arrow + FONT.bold("  Local:   ")
    netwo = arrow + FONT.bold("  Network: ")

    def __init__(self, ip, port) -> None:
        self.ip = ip
        self.port = port

    def __str__(self) -> str:
        url = FONT.blue(f"http://{self.ip}:{self.port}")
        if self.is_localhost(self.ip):
            return self.local + url
        else:
            return self.netwo + url

    @classmethod
    def last_tip(cls) -> str:
        """
        打印最后一条提示信息
        """
        t = FONT.dark_gray("  press ") + FONT.bold(FONT.default("ctrl + c")) + FONT.dark_gray(" to quit")
        return FONT.dark_green(cls._arrow) + t

    @staticmethod
    def is_localhost(ip):
        return ip == "127.0.0.1" or ip == "localhost"

    @staticmethod
    def is_zero_ip(ip):
        return ip == "0.0.0.0"

    @staticmethod
    def get_all_ip() -> list:
        """获取所有可用的ip地址

        Parameters
        ----------
        ip : str
            本机的ip地址

        Returns
        -------
        tuple
            所有可用的ip地址
        """
        interfaces = psutil.net_if_addrs()
        ipv4: list = []
        # APIPA 地址范围
        apipa_range = range(169, 255)
        for _, addresses in interfaces.items():
            for address in addresses:
                # 如果是ipv4地址，且可以被访问到
                if address.family == socket.AddressFamily.AF_INET:
                    # 排除 APIPA 地址范围
                    octets = list(map(int, address.address.split(".")))
                    if octets[0] == 169 and octets[1] in apipa_range:
                        continue
                    ipv4.append(address.address)
        # 对 IPv4 进行排序，"127.0.0.1" 在最前面，剩下按照从小到大排序
        ipv4.sort(key=lambda x: (x != "127.0.0.1", x), reverse=True)
        ipv4.reverse()
        return ipv4



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-02-05 17:03:54
@File: swanlab/db/table_config.py
@IDE: vscode
@Description:
    数据库表格配置
"""
from .models import (
    Project,
    Experiment,
    Tag,
    Chart,
    Namespace,
    Source,
    Display,
)

tables = [Project, Experiment, Tag, Chart, Namespace, Source, Display]



"""
工具函数，用于提供一些常用的工具函数，与项目本身无关，在项目任何文件中都可以使用
"""

from .font import generate_color, FONT, COLOR_LIST
from .file import check_exp_name_format, check_desc_format
from .log import swanlog
from .package import get_package_version

# ---------------------------------- 一些常用但属于外部的东西 ----------------------------------

from swankit.env import get_swanlog_dir



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-06-26 15:14:39
@File: test/unit/db/pytest_swanmodel.py
@IDE: vscode
@Description:
    测试基础数据表模型
"""
import pytest
from tutils import mock_project
from swanboard.db import Project
from swanboard.db.model import SwanModel


class TestSwanModel:
    """测试各个表模型的基础父类"""

    @pytest.fixture(scope="class", autouse=True)
    def setup(self):
        """启动前创建一个项目"""
        project_id = mock_project()
        assert project_id == 1
        yield Project.filter(Project.id == project_id)

    def test_search2dict(self, setup):
        """将查询结果转换为字典"""
        res_dict = SwanModel.search2dict(setup)
        assert isinstance(res_dict, dict)

    def test_search2list(self, setup):
        """将查询结果转换为列表"""
        res_list = SwanModel.search2list(setup)
        assert isinstance(res_list, list)

    def test_json2dict(self):
        """将json字符串转换为字典"""
        res_dict = SwanModel.json2dict('{"key": "value"}')
        assert res_dict == {"key": "value"}

    def test_json2dict_with_empty(self):
        """传递参数为空字符串"""
        res_dict = SwanModel.json2dict("")
        assert res_dict == {}

    def test_dict2json(self):
        """转化字典为json字符串"""
        res_json = SwanModel.dict2json({"key": "value"})
        assert res_json == '{"key": "value"}'

    def test_dict2json_with_json(self):
        """传递参数已经是json字符串"""
        res_json = SwanModel.dict2json('{"key": "value"}')
        assert res_json == '{"key": "value"}'



from fastapi import APIRouter
from ..module.resp import PARAMS_ERROR_422

from ..controller.experiment import (
    # 获取实验信息
    get_experiment_info,
    # 获取 tag 信息
    get_tag_data,
    # 获取实验状态
    get_experiment_status,
    # 获取实验的总结数据
    get_experiment_summary,
    # 获取实验最近日志
    get_recent_logs,
    # 获取实验图标
    get_experimet_charts,
    # 更改实验信息
    update_experiment_info,
    # 删除实验
    delete_experiment,
    # 停止实验
    stop_experiment,
    # 获取实验依赖
    get_experiment_requirements,
    # 修改实验可见性
    change_experiment_visibility,
)
from fastapi import Request
from urllib.parse import quote

router = APIRouter()


@router.get("/{experiment_id}")
def _(experiment_id: int):
    """获取实验信息

    Parameters
    ----------
    experiment_id : int
        实验唯一 id

    Returns
    -------
    dict
        实验信息
    """

    return get_experiment_info(experiment_id)


# COMPAT 由于fastapi不支持%2F的路径转换，所以采用通配符:path，并且在下面将path进行quote编码
@router.get("/{experiment_id}/tag/{tag:path}")
def _(experiment_id: int, tag: str) -> dict:
    """获取表单数据

    parameter
    ----------
    experiment_id: int
        实验唯一id，路径传参
    tag: str
        表单标签，路径传参，使用时需要 URIComponent 编码
    """

    return get_tag_data(experiment_id, tag)


@router.get("/{experiment_id}/status")
def _(experiment_id: int):
    """获取实验状态以及实验图表配置，用于实时更新实验状态

    Parameters
    ----------
    experiment_id : int
        实验唯一id，路径传参
    """

    return get_experiment_status(experiment_id)


@router.get("/{experiment_id}/summary")
def _(experiment_id: int) -> dict:
    """获取实验的总结数据——每个tag的最后一个setp的data

    Parameters
    ----------
    experiment_id : int
        实验id

    Returns
    -------
    dict
        响应信息
    """

    return get_experiment_summary(experiment_id)


@router.get("/{experiment_id}/recent_log")
async def _(experiment_id: int):
    """一下返回最多 MAX_NUM 条打印记录

    Parameters
    ----------
    experiment_id : int
        实验唯一ID
    """

    return get_recent_logs(experiment_id)


@router.get("/{experiment_id}/chart")
def _(experiment_id: int):
    """获取图标信息

    Parameters
    ----------
    experiment_id : int
        实验唯一 ID
    """

    return get_experimet_charts(experiment_id)


@router.patch("/{experiment_id}")
async def _(experiment_id: int, request: Request):
    """修改实验的信息

    Parameters
    ----------
    experiment_id : int
        实验id
    body : Body
        name: str
            实验名称
        description: str
            实验描述

    Returns
    -------
    dict
    """

    return await update_experiment_info(experiment_id, request)


@router.delete("/{experiment_id}")
def _(experiment_id: int):
    """删除实验

    注意，需要先判断当前实验是否正在运行中，不可删除运行中的实验

    Parameters
    ----------
    experiment_id : Int
        实验唯一ID

    Returns
    -------
    project : Dictionary
        删除实验后的项目信息，提供给前端更新界面
    """

    return delete_experiment(experiment_id)


@router.get("/{experiment_id}/stop")
def _(experiment_id: int):
    """停止实验

    Parameters
    ----------
    experiment_id : Int
        实验唯一ID
    """

    return stop_experiment(experiment_id)


# 获取实验依赖
@router.get("/{experiment_id}/requirements")
def _(experiment_id: int):
    """获取实验依赖

    Parameters
    ----------
    experiment_id : int
        实验唯一ID

    Returns
    -------
    requirements: list
        每个依赖项为一行，以列表的形式返回
    """

    return get_experiment_requirements(experiment_id)


# 修改实验可见性
@router.patch("/{experiment_id}/show")
async def _(experiment_id: int, request: Request):
    """修改实验是否可见

    Parameters
    ----------
    experiment_id : int
        实验id
    request : Request
        请求体，包含 show 字段

    Returns
    -------
    experiment : dict
        当前实验信息
    """

    show = (await request.json()).get("show")
    if show is None:
        return PARAMS_ERROR_422("Request parameter 'show'")
    return change_experiment_visibility(experiment_id, show)
